﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Common.Resources;
using HeuristicLab.MainForm;
using System;
using System.Collections.Generic;
using System.Drawing;

namespace HeuristicLab.OptimizationExpertSystem.Menu {
  internal class DownloadFromOKBMenuItem : MenuItemBase {
    public override Image Image { get { return VSImageLibrary.ArrowDown; } }

    public override string Name {
      get { return "Download"; }
    }

    public override IEnumerable<string> Structure {
      get { return new[] { "Config", "OKB" }; }
    }

    public override int Position {
      get { return 30; }
    }

    public override string ToolTipText {
      get {
        return @"Download relevant runs, including algorithm instances
and problem instances from the OKB server.";
      }
    }

    public override void Execute() {
      MainForm.ExpertSystem.UpdateKnowledgeBaseAsync();
    }
    
    protected override void OnToolStripItemSet(EventArgs e) {
      base.OnToolStripItemSet(e);
      ToolStripItem.Enabled = MainForm.ExpertSystem.Problem.ProblemId != -1;
      MainForm.ExpertSystem.Problem.ProblemChanged += OnProblemChanged;
      MainForm.ExpertSystem.DownloadStarted += OnDownloadStarted;
    }

    private void OnProblemChanged(object sender, EventArgs e) {
      ToolStripItem.Enabled = MainForm.ExpertSystem.Problem.ProblemId != -1;
    }

    private void OnDownloadStarted(object sender, EventArgs<IProgress> e) {
      ToolStripItem.Enabled = false;
      e.Value.ProgressStateChanged += DownloadProgressOnStateChanged;
    }

    private void DownloadProgressOnStateChanged(object sender, EventArgs eventArgs) {
      var progress = (IProgress)sender;
      if (progress.ProgressState == ProgressState.Finished || progress.ProgressState == ProgressState.Canceled) {
        ToolStripItem.Enabled = true;
        progress.ProgressStateChanged -= DownloadProgressOnStateChanged;
      }
    }
  }
}
